from invisibleroads_macros_web.markdown import (
    get_html_from_markdown)


def test_get_html_from_markdown():
    html = get_html_from_markdown('x')
    assert not html.startswith('<p>') and not html.endswith('</p>')
    html = get_html_from_markdown('x\n\nx')
    assert html.startswith('<p>') and html.endswith('</p>')
    html = get_html_from_markdown('x\n\n<button></button>\n\nx')
    assert '<p><button></button></p>' not in html
    html = get_html_from_markdown('x\n\n<button></button>\n\n<a></a>.')
    assert '<p><a></a>.</p>' in html



import re

from markdown2 import markdown


SINGLE_PARAGRAPH_PATTERN = re.compile(r'^<p>((?:(?!<p>).)*)</p>$')
NESTED_TAG_PATTERN = re.compile(r'<p>(<.*>)</p>')
EXTRAS = [
    'break-on-newline',
    'code-friendly',
    'cuddled-lists',
    'fenced-code-blocks',
    'footnotes',
    'markdown-in-html',
    'strike',
    'target-blank-links',
    'tables',
    'use-file-vars',
    'task_list']


def get_html_from_markdown(text, extras=EXTRAS):
    html = markdown(text, extras=extras).strip()
    match = SINGLE_PARAGRAPH_PATTERN.match(html)
    if match:
        html = match.group(1)
    return NESTED_TAG_PATTERN.sub(lambda _: _.group(1), html)



from asyncio.exceptions import CancelledError

from starlette.datastructures import MutableHeaders
from starlette.templating import Jinja2Templates


class ExtraResponseHeadersMiddleware:

    def __init__(self, app, headers):
        self.app = app
        self.headers = headers

    async def __call__(self, scope, receive, send):
        if scope['type'] != 'http':
            return await self.app(scope, receive, send)

        async def send_with_extra_headers(message):
            if message['type'] == 'http.response.start':
                headers = MutableHeaders(scope=message)
                for key, value in self.headers.items():
                    headers.append(key, value)

            await send(message)

        try:
            await self.app(scope, receive, send_with_extra_headers)
        except CancelledError:
            pass


class TemplateResponseFactory(Jinja2Templates):

    def __init__(self, environment, context_processors=None):
        'Assume nothing about the template environment'
        self.env = environment
        self.context_processors = context_processors or []



from invisibleroads_macros_web.escape import (
    escape_quotes_html,
    escape_quotes_js)


def test_escape_quotes_html():
    assert escape_quotes_html(1) == 1
    assert escape_quotes_html('"' + "'") == '&#34;&#39;'


def test_escape_quotes_js():
    assert escape_quotes_js(1) == 1
    assert escape_quotes_js('"' + "'") == '\\"' + "\\'"



import socket

from pytest import raises

import invisibleroads_macros_web.port as module
from invisibleroads_macros_web.port import (
    find_open_port,
    is_port_in_use)


def test_find_open_port(monkeypatch):
    monkeypatch.setattr(
        module, 'randint', lambda a, b: 5000)
    monkeypatch.setattr(
        module, 'is_port_in_use', lambda x: False)
    assert find_open_port() == 5000
    assert find_open_port(7000) == 7000
    monkeypatch.setattr(
        module, 'is_port_in_use', lambda x: True if x == 7000 else False)
    with raises(OSError):
        find_open_port(7000, 7000, 7000)
    assert find_open_port(7000) == 5000


def test_is_port_in_use():
    port = 65432
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        s.bind(('127.0.0.1', port))
        s.listen()
        assert is_port_in_use(port)
    assert not is_port_in_use(port)
